@@ -93,6 +93,10 @@
 #'
 #'  The defaults for both arguments is \code{"template"}, to use the same option
 #'  used for estimation of the \code{template}.
+#' @param covariates Numeric vector of covariates to take into account for model
+#'  estimation. Names should give the name of each variable. The covariates must
+#'  match those of the template. Default: \code{NULL} (no covariates).
+#'  NOTE: Not implemented yet.
 #' @param brainstructures Only applies if the entries of \code{BOLD} are CIFTI
 #'  file paths. This is a character vector indicating which brain structure(s)
 #'  to obtain: \code{"left"} (left cortical surface), \code{"right"} (right
@@ -215,6 +219,7 @@ templateICA <- function(
   GSR="template",
   Q2="template",
   Q2_max="template",
+  covariates=NULL,
   brainstructures="template",
   mask=NULL,
   time_inds=NULL,
@@ -267,6 +272,30 @@ templateICA <- function(
   }
   if (Q2 == "template") { Q2 <- template$params$Q2 }
   if (Q2_max == "template") { Q2_max <- template$params$Q2_max }
+  if (!is.null(template$params$covariate_names)) {
+    covariate_names <- template$params$covariate_names
+    nC <- length(covariate_names)
+    if (is.null(covariates)) {
+      stop("These covariates were used during template estimation: ", 
+        paste0(covariate_names, collapse=", "), ". They must also be provided ", 
+        "to `templateICA` with the `covariates` argument.")
+    }
+    stopifnot(is.numeric(covariates) && is.vector(covariates))
+    stopifnot(length(covariates) == length(covariate_names))
+    if(!all(names(covariates) == covariate_names)) {
+      stop("These covariates were used during template estimation: ", 
+        paste0(covariate_names, collapse=", "), ". The same covariates must ", 
+        "also be provided to `templateICA` with the `covariates` argument. ",
+        "However, the names for `covariates` provided here differ.")
+    }
+  } else {
+    if (!is.null(covariates)) { 
+      stop("`covariates` is not `NULL`, yet none were used during template ",
+      "estimation. Any covariates must match those used in the template.")
+    }
+    covariate_names <- NULL
+    nC <- 0
+  }
   brainstructures <- match.arg(
     brainstructures,
     c("template", "left", "right", "subcortical", "all"),
@@ -298,9 +327,9 @@ templateICA <- function(
   if (isFALSE(spatial_model)) { spatial_model <- NULL }
   if (!is.null(resamp_res)) {
     stopifnot(is_posNum(resamp_res) && round(resamp_res) == resamp_res)
-      if (!requireNamespace("ciftiTools", quietly = TRUE)) {
-        stop("Package \"ciftiTools\" needed to work with CIFTI data. Please install it.", call. = FALSE)
-      }
+    if (!requireNamespace("ciftiTools", quietly = TRUE)) {
+      stop("Package \"ciftiTools\" needed to work with CIFTI data. Please install it.", call. = FALSE)
+    }
   }
   stopifnot(is_1(rm_mwall, "logical"))
   stopifnot(is_1(reduce_dim, "logical"))
@@ -377,10 +406,30 @@ templateICA <- function(
     do_left <- "left" %in% brainstructures
     do_right <- "right" %in% brainstructures
     do_sub <- "subcortical" %in% brainstructures
+
+    if (format == "xifti") {
+      if (!requireNamespace("ciftiTools", quietly = TRUE)) {
+        stop("Package \"ciftiTools\" needed to work with xifti data. Please install it.", call. = FALSE)
+      }
+
+      for (bb in seq(nN)) {
+        if (!do_left && !is.null(BOLD[[bb]]$data$cortex_left)) {
+          BOLD[[bb]] <- ciftiTools::remove_xifti(BOLD[[bb]], "cortex_left")
+        }
+        if (!do_right && !is.null(BOLD[[bb]]$data$cortex_right)) {
+          BOLD[[bb]] <- ciftiTools::remove_xifti(BOLD[[bb]], "cortex_right")
+        }
+        if (!do_sub && !is.null(BOLD[[bb]]$data$subcort)) {
+          BOLD[[bb]] <- ciftiTools::remove_xifti(BOLD[[bb]], "subcortical")
+        }
+      }
+    }
   }
 
   # Read in CIFTI, GIFTI, or NIFTI files.
-  if(verbose) cat("Reading BOLD data.\n")
+  if (verbose && format %in% c("CIFTI", "GIFTI", "NIFTI", "RDS")) {
+    cat("Reading BOLD data.\n")
+  }
 
   # (Need to do now rather than later, so that CIFTI resolution info can be used.)
   if (format == "CIFTI") {
@@ -1147,6 +1196,7 @@ templateICA <- function(
     time_inds=time_inds, GSR=GSR,
     scale=scale, TR=TR, hpf=hpf,
     Q2=Q2, Q2_max=Q2_max, Q2_est=Q2_est,
+    covariate_names=covariate_names,
     brainstructures=brainstructures,
     tvar_method=tvar_method,
     spatial_model=do_spatial,
